
#ifndef _LDFS_LCACHE_H
#define _LDFS_LCACHE_H

#include <linux/module.h>
#include <linux/string.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <asm/atomic.h>

#include "ext4.h"
#include "sdfs_slist.h"
#include "sdfs_common.h"

#define LDFS_LCACHE_GROUP_COUNT 32
#define LDFS_LCACHE_SEARCH_HEAD 8192

#define LDFS_LCACHE_LOCK_CNT 256
#define LDFS_LCACHE_LOCK(fp) (fp[0])

#define LDFS_LCACHE_DIV_CNT 64
#define LDFS_LCACHE_DIV_WIDTH(sb) (LDFS_STORE_ENTRIES(sb) / LDFS_LCACHE_DIV_CNT)
#define LDFS_LCACHE_ENTRY_POS(fp) ((fp[1] << 8 | fp[0]) % LDFS_LCACHE_SEARCH_HEAD)

#define LDFS_LCACHE_SMP

struct ldfs_lcache_entry {
	struct slist_head e_head;
	__u8 e_fp[LDFS_FP_LEN_MAX];
};

struct ldfs_lcache_group {
	struct list_head g_head;
	ext4_fsblk_t g_groupnr;
	atomic_t g_count;
	atomic_t g_load_div;
	int valid;
	struct mutex g_load_mutex;
	struct slist_head g_entry_head[LDFS_LCACHE_SEARCH_HEAD];
	struct ldfs_lcache_entry * g_entry;
};

struct ldfs_lcache {
	struct super_block * l_sb;
	struct list_head l_using_head;
	struct list_head l_wait_head;
	struct list_head l_reload_head;
	struct list_head l_free_head;
	spinlock_t l_group_lock;
	spinlock_t l_entry_lock[LDFS_LCACHE_LOCK_CNT];
};

typedef int (*insert_fp_t)(struct super_block * sb, struct ldfs_lcache_group * cache_group,
		ext4_fsblk_t blocknr, __u8 fp[], __u32 cnt);


int ldfs_lcache_search_group(struct super_block * sb, __u8 fp[],
		ext4_fsblk_t groupnr, ext4_fsblk_t * blocknr);
int ldfs_lcache_search(struct super_block * sb, __u8 fp[], ext4_fsblk_t * blocknr);
int ldfs_lcache_guided_search(struct super_block * sb, __u8 fp[], ext4_fsblk_t guide_blocknr, ext4_fsblk_t * blocknr);

int ldfs_lcache_invalidate(struct super_block * sb, ext4_fsblk_t blocknr);

int ldfs_lcache_init(struct super_block * sb);
int ldfs_lcache_exit(struct super_block * sb);

#endif /* _LDFS_LCACHE_H */
