
#ifndef _LDFS_STORE_H
#define _LDFS_STORE_H

#include <linux/module.h>
#include <linux/buffer_head.h>
#include <linux/string.h>
#include <linux/fs.h>
#include "ext4.h"
#include "ext4_jbd2.h"
#include "sdfs_common.h"
#include "sdfs_lcache.h"

#define LDFS_S_FP_LEN 28
#define LDFS_S_CNT 128
#define LDFS_STORE_COMPACT 4L
#define LDFS_STORE_ENTRIES(sb) (EXT4_BLOCKS_PER_GROUP(sb) * LDFS_STORE_COMPACT)

struct ldfs_s_entry {
	__u32 count;
	__u32 offset;
	__u8 fp[24];		// Temporary change
};

struct ldfs_store_fetch_ops {
	int (*ldfs_count_fn)(struct super_block * sb, ext4_fsblk_t blocknr, __u8 fp[], void * data);
	int (*ldfs_nocount_fn)(struct super_block * sb, ext4_fsblk_t blocknr, __u8 fp[], void * data);
	void * data;
};

ext4_fsblk_t ldfs_get_store_position(struct super_block * sb, ext4_fsblk_t blocknr, __u32 * offset);
int ldfs_store_prefetch(struct super_block * sb, ext4_group_t group);
int ldfs_store_read(struct super_block * sb, ext4_fsblk_t blocknr, __u8 fp[], __u32 * cnt);
int ldfs_store_write(handle_t * handle, struct inode * inode, ext4_fsblk_t blocknr, __u8 fp[], __u32 cnt);
int ldfs_store_fp_cmp(struct inode * inode, ext4_fsblk_t blocknr, __u8 fp[]);
int ldfs_store_count_link(handle_t * handle, struct inode * inode, ext4_fsblk_t blocknr);
int ldfs_store_count_unlink(handle_t * handle, struct inode * inode, ext4_fsblk_t blocknr, __u8 fp[], __u32 * cnt);
int ldfs_store_fetch(struct super_block * sb, ext4_fsblk_t start_blocknr, __u32 cnt,
		insert_fp_t insert_fp, struct ldfs_lcache_group * cache_group);
int ldfs_store_mount(struct super_block * sb);
#endif /* _LDFS_STORE_H */
